/************ ANLAGE A *********************/
package algo;

import javax.swing.JTable;
import javax.swing.table.*;
import java.awt.*;
import java.util.*;
import gui.ColorRenderer;
/*
 * Created on 12.11.2005
 * Implementierung Simplex-Algorithmus
 * 	5 | 8  | 700
 *  1 | 1  | 100
 *  0 | 1  | 60
 * - - - - - - -
 * -1 | -2 | 0
 */

/**
 * @author Sibel Cilek und Daniela Zielke, AI7
 */
public class Simplex {

	private static int anzZeilen;
	private static int anzSpalten;
	private static int minMaxProblem;
	// 2-dimensionales Array fr Tableau-Darstellung
	private static double tableau[][], pivotElement=0.0;
	private static int pivotZeile, pivotSpalte;
	private static ArrayList listTableaus = new ArrayList();
	private static double x[][];
	private static int anzIterationen;
	private static boolean gleich;
		
	public Simplex(){
		minMaxProblem = 0;
		pivotSpalte = 0;
		pivotZeile = 0;
		anzZeilen = 0;
		anzSpalten = 0;
		anzIterationen=0;
		gleich = false;
		listTableaus.clear();
	}
	
	/**
	 * Startet die Berechnung der Aufgabe 
	 * nach dem Simplex-Algorithmus
	 */
	public void starteRechnung(){
		boolean negativ = true;
		x = new double[anzSpalten-1][2];
		/* Prfe, ob Minimierungsaufgabe */
		if(minMaxProblem == 1){
			transformTableau();
		}
		setTable();
		/*Start Iteration */
		while(negativ){
			/* Anzahl der Iterationen erhhen */
			anzIterationen++;
			this.setAnzIterationen(anzIterationen);
			pivotSpalte = 0;
			pivotZeile = 0;
			/* Berechne Pivot-Element */
			findePivotElement();
			/* Berechne restliche Element */
			berechneElemente();
			/* Vorbereitung fr Ausgabe in GUI */
			setTable();
			negativ = false;
			/* Prfe, ob es noch negative Zielkoeffizienten gibt */
			for(int j=0;j<anzSpalten-1;j++){
				if(tableau[anzZeilen-1][j] < 0)
					negativ = true;
			}
		}
		
	}
	/**
	 * Methode transformiert das Tableau,
	 * wenn die Aufgabe ein Minimierungs-
	 * problem ist
	 */
	public void transformTableau(){
		double[][] temp = tableau;
		int tempAnz = 0;
		tempAnz = anzSpalten;
		anzSpalten = anzZeilen;
		anzZeilen = tempAnz;
		tableau = new double[anzZeilen][anzSpalten+(tempAnz-1)];
		for(int i=0;i<anzZeilen;i++){
			for(int j=0;j<anzSpalten;j++){
				if(j >= anzZeilen){
					tableau[i][j] = 0;
				}
				if((i == anzZeilen-1)||(j == anzSpalten-1))
					tableau[i][j] = -temp[j][i];
				else
					tableau[i][j] = temp[j][i];
			}
			int tempZeilen = anzZeilen;
			if(anzZeilen == anzSpalten){
				tempZeilen--;
				gleich = true;
			}
			tableau[i][anzSpalten+1] = tableau[i][tempZeilen];
			tableau[i][tempZeilen] = 0;
			if((tempZeilen+i) != (anzSpalten+1))
				tableau[i][tempZeilen+i] = 1;
		}
		anzSpalten = anzSpalten+(tempAnz-1);
	}
		
	/**
	 * Methode, die das Pivotelement
	 * im Tableau bestimmt
	 */
	public void findePivotElement(){
		double maxWert=0.0;
		double minDivision=0.0;
		double Division=0.0;
		for(int j=0;j<anzSpalten-1;j++){
			if(tableau[anzZeilen-1][j] < maxWert){
				maxWert = tableau[anzZeilen-1][j];
				pivotSpalte = j;
			}
		}
		for(int i=0;i<anzZeilen-1;i++){
			if(tableau[i][pivotSpalte] > 0){
				Division = (tableau[i][anzSpalten-1])/(tableau[i][pivotSpalte]);
				if((Division < minDivision) || (minDivision == 0)){
					minDivision = Division;
					pivotZeile = i;
				}
			}
		}
		pivotElement = tableau[pivotZeile][pivotSpalte];
	
	}
	
	/**
	 * Methode rechnet das neue Tableau aus
	 */
	public void berechneElemente(){
		/* Berechnung der restlichen Elemente */
		for(int i=0;i<anzZeilen;i++){
			for(int j=0;j<anzSpalten;j++){
				if((i!=pivotZeile)&&(j!=pivotSpalte)){
					tableau[i][j] = tableau[i][j]-((tableau[i][pivotSpalte]*tableau[pivotZeile][j])/pivotElement);
				}
			}
		}
		
		/* alle Spalten in der Pivotzeile durch Pivotelement teilen */
		for(int j=0;j<anzSpalten;j++){
			tableau[pivotZeile][j]=tableau[pivotZeile][j]/pivotElement;
		}
		
		/* alle Zeilen in der Pivotspalte auf 0 setzen */
		for(int i=0;i<anzZeilen;i++){
			if(i != pivotZeile){
				tableau[i][pivotSpalte] = 0;
			}
		}
	}
	
	
	public void setTable(){
		JTable table = new JTable(anzZeilen+1,anzSpalten+1);
		String s = "";
		int temp = 0;
		int zaehl = anzSpalten;
		TableCellRenderer ren = new ColorRenderer((pivotZeile+1),(pivotSpalte+1),anzIterationen);
		table.setDefaultRenderer(Object.class,ren);
		if(minMaxProblem == 1){
			temp = anzZeilen;
			if(gleich)
				temp--;
			for(int i=0;i<temp;i++){
				s = "x"+(i+temp);
				table.setValueAt(s,0,(i+1));
			}		
			zaehl = anzZeilen;
		}
		for(int i=1;i<zaehl;i++){
			s = "x"+i;
			table.setValueAt(s,0,(i+temp));
		}	
		table.setValueAt("b",0,anzSpalten);
		if(minMaxProblem == 0)
			temp = anzSpalten-1;
		else
			temp = 0;			
		for(int j=1;j<anzZeilen;j++){
			s = "x"+(temp+j);
			table.setValueAt(s,j,0);
		}
		table.setValueAt("F",anzZeilen,0);
		for(int i=0;i<anzZeilen;i++){
			for(int j=0;j<anzSpalten;j++){
				table.setValueAt(new Double(Math.round(tableau[i][j]*100.0)/100.0),i+1,j+1);
			}
		}
		if((anzIterationen > 0) && (minMaxProblem == 0)){
			int z = 0;
			x[pivotSpalte][0] = ((Double)(table.getValueAt((pivotZeile+1),anzSpalten))).doubleValue();
			x[pivotSpalte][1] = pivotZeile+1;
			s = "x"+(pivotSpalte+1);
			z = (int)x[pivotSpalte][1];
			table.setValueAt(s,z,0);
			for(int k=0;k<anzSpalten-1;k++){
				if(x[k][0] > 0.0){
					z = (int)x[k][1];
					if(z != (pivotZeile+1)){
						x[k][0] = ((Double)(table.getValueAt(z,anzSpalten))).doubleValue();
						s = "x"+(k+1);
						table.setValueAt(s,z,0);
					}
					if((z == (pivotZeile+1)) && (k != pivotSpalte)){
						x[k][0] = 0.0;
						x[k][1] = 0.0;
					}
				}
			}
		}
		if((anzIterationen > 0) && (minMaxProblem == 1)){
			for(int k=0;k<anzZeilen-1;k++){
				if(gleich)
					x[k][0] = ((Double)(table.getValueAt(anzZeilen,(k+anzZeilen)))).doubleValue();
				else
					x[k][0] = ((Double)(table.getValueAt(anzZeilen,(k+1+anzZeilen)))).doubleValue();
			}
		}
		table.setShowHorizontalLines(false);
		table.setShowVerticalLines(false);
		table.setGridColor(Color.gray);
		table.setShowGrid(true);
		table.setRowHeight(20);
		table.setVisible(true);
		listTableaus.add(table);
	}
	
	/**
	 * @return Returns the tableau.
	 */
	public double[][] getTableau() {
		return tableau;
	}

	/**
	 * @param anzSpalten The anzSpalten to set.
	 */
	public void setAnzSpalten(int anzSpalten) {
		this.anzSpalten = anzSpalten;
	}

	/**
	 * @param anzZeilen The anzZeilen to set.
	 */
	public void setAnzZeilen(int anzZeilen) {
		this.anzZeilen = anzZeilen;
	}

	/**
	 * @param minMaxProblem The minMaxProblem to set.
	 */
	public void setMinMaxProblem(int minMaxProblem) {
		this.minMaxProblem = minMaxProblem;
	}
	/**
	 * @param tableau The tableau to set.
	 */
	public void setTableau(double[][] tableau) {
		this.tableau = new double[anzZeilen][anzSpalten];
		this.tableau = tableau;
	}
	/**
	 * @return Returns the anzSpalten.
	 */
	public int getAnzSpalten() {
		return anzSpalten;
	}
	/**
	 * @return Returns the anzZeilen.
	 */
	public int getAnzZeilen() {
		return anzZeilen;
	}
	
	/**
	 * @return Returns the anzIterationen.
	 */
	public int getAnzIterationen() {
		return anzIterationen;
	}

	/**
	 * @param anzIterationen The anzIterationen to set.
	 */
	public void setAnzIterationen(int anzIterationen) {
		this.anzIterationen = anzIterationen;
	}

	/**
	 * @return Returns the listTableaus.
	 */
	public ArrayList getListTableaus() {
		return listTableaus;
	}

	/**
	 * @return Returns the x.
	 */
	public double[][] getX() {
		return x;
	}

}
